/*! @addtogroup GPIO General Purpose Input/Output
 *  @{
 *  @ingroup 	DEVICE_DRIVERS 
 *  @file    	gpio.c
 *	@brief		GPIO driver for Freescale Kinetis L - series devices
 * @}
 */
#include "gpio.h"

/*!
**     @brief
**         The specified output value is set to the pin.
**     @param gpio	-  GPIOx Device data structure pointer
**     @param pin   -  Bit/Pin number
**     @param val   -  A new bit value. Possible values:
**                     <false> - logical "0" (Low level)
**                     <true> - logical "1" (High level)  
***/    
void pin_PutVal(GPIO_MemMapPtr gpio, uint8_t pin, bool val)
{
	if(val)
		GPIO_PSOR_REG(gpio) |= PIN(pin);
	else
		GPIO_PCOR_REG(gpio) |= PIN(pin);
}
/*!
**     @brief
**         Clears (set to zero) the output value. It is equivalent to
**         the [PutVal(FALSE)].
**     @param gpio	-  GPIOx Device data structure pointer
**     @param pin   -  Bit/Pin number
**     
***/
void pin_ClrVal(GPIO_MemMapPtr gpio, uint8_t pin)
{
	GPIO_PCOR_REG(gpio) |= PIN(pin);
}
/*!
**     @brief
**         Sets (to one) the output value. It is equivalent to the
**         [PutVal(TRUE)]. 
**     @param gpio	-  GPIOx Device data structure pointer
**     @param pin   -  Bit/Pin number
*/
void pin_SetVal(GPIO_MemMapPtr gpio, uint8_t pin)
{
	GPIO_PSOR_REG(gpio) |= PIN(pin);
}
/*!
**     @brief
**         Negates (inverts) the output value. 
**     @param gpio	-  GPIOx Device data structure pointer
**     @param pin   -  Bit/Pin number
*/
void pin_NegVal(GPIO_MemMapPtr gpio, uint8_t pin)
{
	GPIO_PTOR_REG(gpio) |= PIN(pin);
}
/*!
**     @brief
**         Returns the input/output value.
**     @param gpio	-  GPIOx Device data structure pointer 
**     @param pin   -  Bit/Pin number
**     
*/
bool pin_GetVal(GPIO_MemMapPtr gpio, uint8_t pin)
{
	return ((GPIO_PDIR_REG(gpio) & PIN(pin)) >> pin);
}
/*!
**     @brief
**         Specified value is passed to the PORT. 
**     @param gpio	-  GPIOx Device data structure pointer
**     @param Val   -  Output value
*/
void port_PutVal(GPIO_MemMapPtr gpio, uint32_t val)
{
	GPIO_PDOR_REG(gpio) = val;
}
/*!
**     @brief
**         Returns the value of the Port
**     @param gpio	-  GPIOx Device data structure pointer
**     @return - Input value
*/
uint32_t port_GetVal(GPIO_MemMapPtr gpio)
{
	return GPIO_PDIR_REG(gpio);
}
