/*! @addtogroup SYSTEM System Initialization
 *  @{
 *  @file    	sysinit.c
 *	@brief		System initialization for FRDM-KL25Z board
 * 
 * @}
 */

#include "sysinit.h"

/*! MCG Output (Hz) MCGOUTCLK*/
uint32_t mcg_clk_hz;
/*! MCG Output (kHz) MCGOUTCLK*/
uint32_t mcg_clk_khz;
/*!MCGOUTCLK divided by OUTDIV1, clocks the ARM Cortex- M0+ core*/
uint32_t core_clk_khz;
/*! System clock divided by OUTDIV4, clocks the bus slaves and peripherals */
uint32_t periph_clk_khz;
/*! MCG PLL Clock (kHz)*/
uint32_t pll_clk_khz;
/*! UART0 Clock (kHz)*/
uint32_t uart0_clk_khz;
/*!UART0 Clock (Hz)*/
uint32_t uart0_clk_hz;

/*! @brief Initialize PLL, Pins Ports and UART modules.
 * 
 */
void sysinit(void)
{

	/*Pin Initialization*/
	port_init(pin_definitions);
	
	/* Ramp up the system clock */
	/* Set the system dividers */
	/* NOTE: The PLL init will not configure the system clock dividers,
	* so they must be configured appropriately before calling the PLL
	* init function to ensure that clocks remain in valid ranges.
	*/  
	SIM_CLKDIV1 = ( 0
					| SIM_CLKDIV1_OUTDIV1(0)
					| SIM_CLKDIV1_OUTDIV4(1));

#if PLL_INIT == 0
        // If PLL initialization is not desired, set FLL to 48 MHz clock in default FEI mode
        //MCG_C4 |= (MCG_C4_DRST_DRS(1) | MCG_C4_DMX32_MASK);
        mcg_clk_hz = 21000000; //FEI mode
        
        SIM_SOPT2 &= ~SIM_SOPT2_PLLFLLSEL_MASK; // clear PLLFLLSEL to select the FLL for this clock source
        
        uart0_clk_khz = (mcg_clk_hz / 1000); // the uart0 clock frequency will equal the FLL frequency
#else
	
	/* Initialize PLL */
	/* PLL will be the source for MCG CLKOUT so the core, system, and flash clocks are derived from it */ 
	mcg_clk_hz = pll_init(	 CLK0_FREQ_HZ,  /* CLKIN0 frequency */
							 LOW_POWER,     /* Set the oscillator for low power mode */
							 CLK0_TYPE,     /* Crystal or canned oscillator clock input */
							 PLL0_PRDIV,    /* PLL predivider value */
							 PLL0_VDIV,     /* PLL multiplier */
							 PLLS);         /* Use the output from this PLL as the MCGOUT */
	
	/*
	 * Use the value obtained from the pll_init function to define variables
	 * for the core clock in kHz and also the peripheral clock. These
	 * variables can be used by other functions that need awareness of the
	 * system frequency.
	 */
	mcg_clk_khz = mcg_clk_hz / 1000;
	core_clk_khz = mcg_clk_khz / (((SIM_CLKDIV1 & SIM_CLKDIV1_OUTDIV1_MASK) >> 28)+ 1);
	periph_clk_khz = core_clk_khz / (((SIM_CLKDIV1 & SIM_CLKDIV1_OUTDIV4_MASK) >> 16)+ 1);
	
	SIM_SOPT2 |= SIM_SOPT2_PLLFLLSEL_MASK; // set PLLFLLSEL to select the PLL for this clock source
#endif
	
	systick_init(core_clk_khz);
	
	lcd_init();
	
	accel_init();
	
#if TERM_PORT_NUM == 0
		SIM_SOPT2 |= SIM_SOPT2_UART0SRC(1); // select the PLLFLLCLK as UART0 clock source
		uart0_clk_khz = ((mcg_clk_hz / 2) / 1000); // UART0 clock frequency will equal half the PLL frequency
		uart0_init (UART0_BASE_PTR, uart0_clk_khz, TERMINAL_BAUD, POLLING);  
#elif TERM_PORT_NUM == 1
		uart_init (UART1_BASE_PTR, periph_clk_khz, TERMINAL_BAUD, POLLING);
#else
		uart_init (UART2_BASE_PTR, periph_clk_khz, TERMINAL_BAUD, POLLING);
#endif
		
		
}
