/*! @file	pin_assig.c
 *  @brief Pin Assignment File.
 *  @details This is file assigns functionality to each pin in the pin_definitions array. 
 */
#include "pin_assig.h"

typedef enum
{
	EN = 0,
	RS,
	D4,
	D5,
	D6,
	D7,
	UART0_RX,
	UART0_TX,
    TOTAL_GPIO_PINS
}gpio_pin_name_t;

const pin_struct_t pin_definitions[TOTAL_GPIO_PINS] = 
{
    /*PIN NAME		  | port | pin |  irq |  mux |  pe |  ds  | mode | value */
	/* EN		  	*/ { PTA,   4, DISABLED, ALT1,  TRUE, FALSE, OUTPUT,  0 },
	/* RS		  	*/ { PTA,   5, DISABLED, ALT1,  TRUE, FALSE, OUTPUT,  0 },
	/* D4			*/ { PTB,   0, DISABLED, ALT1, 	TRUE, FALSE, OUTPUT,  0 },
    /* D5			*/ { PTB,   1, DISABLED, ALT1, 	TRUE, FALSE, OUTPUT,  0 },
    /* D6			*/ { PTB,   2, DISABLED, ALT1, 	TRUE, FALSE, OUTPUT,  0 },
    /* D7			*/ { PTB,   3, DISABLED, ALT1, 	TRUE, FALSE, OUTPUT,  0 },
    /* UART0_RX  	*/ { PTA,   1, DISABLED, ALT2,  TRUE, FALSE,   NULL,  0 },
    /* UART0_TX  	*/ { PTA,   2, DISABLED, ALT2,  TRUE, FALSE,   NULL,  0 },
};


/*! @brief Initialize Pin Signals according to pin_definitions array
 * 
 * @param pin_def	- Receive pin_definitions array
 */
void port_init(const pin_struct_t pin_def[])
{
	gpio_pin_name_t i;
	GPIO_MemMapPtr GPIO;
	PORT_MemMapPtr PORT;
	
	/* Enable all of the port clocks. These have to be enabled to configure
	* pin muxing options, so most code will need all of these on anyway.
	*/
	SIM_SCGC5 |= (SIM_SCGC5_PORTA_MASK
			  | SIM_SCGC5_PORTB_MASK
			  | SIM_SCGC5_PORTC_MASK
			  | SIM_SCGC5_PORTD_MASK
			  | SIM_SCGC5_PORTE_MASK );
	
	for(i=0; i < TOTAL_GPIO_PINS; i++)
	{
		/* Get the Port base pointer*/
		PORT = pin_def[i].port; 
		/* Get the GPIO base pointer*/
		GPIO = (GPIO_MemMapPtr)((((uint32_t)PORT - PORTn_BASE)>>6 ) + GPIOn_BASE);
		/* Set  PORTx_PCRn Pin Control Register */
		PORT_PCR_REG(PORT,pin_def[i].pin) =	PORT_PCR_IRQC(pin_def[i].irq) /* Interrupt Configuration */
											|PORT_PCR_MUX(pin_def[i].mux);/* Pin Mux Control */
		/* 	Pull enable */								
		if(pin_def[i].pe)
				PORT_PCR_REG(PORT,pin_def[i].pin) 	|= PORT_PCR_PE_MASK
													|PORT_PCR_PS_MASK;
		/* Drive Strength Enable*/								
		if(pin_def[i].ds)
				PORT_PCR_REG(PORT,pin_def[i].pin) 	|= PORT_PCR_DSE_MASK;
													
		
		/* If Pin Signal is configuer as GPIO*/
		if(pin_def[i].mux == ALT1)
		{
			/* Initial Value */
			if(pin_def[i].value)
				GPIO_PDOR_REG(GPIO) |= PIN_SHIFT(pin_def[i].pin);
			else
				GPIO_PDOR_REG(GPIO) &= ~(PIN_SHIFT(pin_def[i].pin));
			/* Configure Input or Output mode */
			if(pin_def[i].mode)
				GPIO_PDDR_REG(GPIO) |= PIN_SHIFT(pin_def[i].pin); 
			else
				GPIO_PDDR_REG(GPIO) &= ~(PIN_SHIFT(pin_def[i].pin));
		}
	}
}
